# fca.py
#
# This file is part of python wrapper for the BROADWAY2 package.
#
# MIT License

# Copyright (c) 2013-2020 FibreCode GmbH <info@fibrecode.com>

# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:

# The above copyright notice and this permission notice shall be included in all
# copies or substantial portions of the Software.

# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.

import binascii
from broadway2 import broadway_api

class FCProductId:
    FW2ProductId = {'Broadway_FC300104': 'FC611',
                    'Broadway_FC300105': 'FC602',
                    'Broadway_FC300106': 'FC612',
                    'Broadway_FC300107': 'FC401',
                    'Broadway_FC300108': 'FC621',
                    'Broadway_FC300109': 'FC631'
                    }

class FCAdapter:
    def __init__(self):
        self.__device = None     # store connected control device (connect/disconnect)
        self.Inst = -1
        self.DevName = ''
        self.UsbVendorId = -1
        self.UsbProductId = -1
        self.UsbRevision = -1
        self.FWName = ''
        self.FWRevision = ''
        self.SerialNumber = ''
        self.MACAddress = ''
        self.BoardId = ''
        self.Features = {'Raw': False, 'Tool': False}
        self.PhyType = ''
        self.MtpVersionReadable = False
        self.FriendlyName = ''

    def initialize(self, inst, dev):
        """initialize instance with properties from given dev"""
        self.Inst = inst
        if dev == None:
            self.DevName = 'UNKNOWN'
            self.DeviceType = 0
            self.SerialNoBase = 'n/a'
            self.MACAddress = 'n/a'
            self.PhyType = 'n/a'
            self.FWRevision = 'n/a'
            return

        res = broadway_api.get_device_property_uint(dev, broadway_api.FCBDeviceProperty.FCBDevicePropertyUsbVendorId)
        self.UsbVendorId = res
        res = broadway_api.get_device_property_uint(dev, broadway_api.FCBDeviceProperty.FCBDevicePropertyUsbProductId)
        self.UsbProductId = res
        res = broadway_api.get_device_property_uint(dev, broadway_api.FCBDeviceProperty.FCBDevicePropertyUsbRevBcd)
        self.UsbRevision = res
        res = broadway_api.get_device_serial_number(dev)
        self.SerialNumber = res
        self.SerialNoBase = broadway_api.get_device_property_uint(dev, broadway_api.FCBDeviceProperty.FCBDevicePropertySerialNoBase)
        self.SerialNoAddOn = broadway_api.get_device_property_uint(dev, broadway_api.FCBDeviceProperty.FCBDevicePropertySerialNoAddOn)
        self.FWName = broadway_api.get_device_property_string(dev, broadway_api.FCBDeviceProperty.FCBDevicePropertyFwVariantName)
        try:
            self.DevName = FCProductId.FW2ProductId[self.FWName]
        except:
            self.DevName = 'UNKNOWN/NOT SUPPORTED'
        self.FWRevision = broadway_api.get_device_property_string(dev, broadway_api.FCBDeviceProperty.FCBDevicePropertyFwVerString)
        self.PhyType = broadway_api.get_device_property_string(dev, broadway_api.FCBDeviceProperty.FCBDevicePropertyPhyTypeString)
        try:
            if broadway_api.get_device_property_uint(dev, broadway_api.FCBDeviceProperty.FCBDevicePropertyMtpVersionReadable) > 0:
                self.MtpVersionReadable = True
        except:
            self.MtpVersionReadable = False

        mac = broadway_api.get_device_property_binary(dev, broadway_api.FCBDeviceProperty.FCBDevicePropertyMacAddress)
        self.MACAddress = ''
        try:
            for m in mac:
                self.MACAddress += '{0:02X}'.format(m)
        except:
            pass
 
        skey = broadway_api.get_device_property_binary(dev, broadway_api.FCBDeviceProperty.FCBDevicePropertySiliconKey)
        self.SiliconKey = ''
        try:
            for sk in skey:
                self.SiliconKey += '{0:02X}'.format(sk)
        except:
            pass

        self.FriendlyName = broadway_api.get_device_property_string(dev, broadway_api.FCBDeviceProperty.FCBDevicePropertyFriendlyName)

        tm = broadway_api.get_timestamp(dev)

        self.StickTime = int(tm[0]*1e9+tm[1])
        

        # Features based on device name

        if self.DevName == 'FC611' or self.DevName == 'FC612' or self.DevName == 'FC631':
            self.Features['Raw'] = True
        if self.DevName == 'FC401':
            self.Features['Tool'] = True

        # new supported on broadway2-API > 2.13
        try:
            self.DeviceType = broadway_api.get_device_property_uint(dev, broadway_api.FCBDeviceProperty.FCBDevicePropertyDeviceType)
        except:
            self.DeviceType = None # unknown as older API. Stay backward-compatible


    def connect(self, dev):
        """connect the FC6XX instance to connected hardware adapter (control-API)"""
        self.__device = dev

    def disconnect(self):
        """disconnect FC6xx from current connected hardware instance"""
        self.__device = None

    def is_connected(self):
        """retrive connection status: True/False"""
        return self.__device != None

    def is_raw_enabled(self):
        """only FC611/FC612 supports Layer-2 raw-interface, not acting as network device"""
        if self.DevName == 'FC611' or self.DevName == 'FC612' or self.DevName == 'FC631':
            return true
        return false

    @property
    def Device(self):
        return self.__device
    
    @property
    def PhyMaster(self):
        return broadway_api.get_device_property_uint(self.__device, broadway_api.FCBDeviceProperty.FCBDevicePropertyMasterSlave) == broadway_api.FCBPhyMode.FCBMasterMode

    @PhyMaster.setter
    def PhyMaster(self, value):
        """encapsulated property to keep Stick-state sync, e.g. LED color"""
        if value:
            setMode = broadway_api.FCBPhyMode.FCBMasterMode
        else:
            setMode = broadway_api.FCBPhyMode.FCBSlaveMode

        broadway_api.set_device_property_uint(self.__device, broadway_api.FCBDeviceProperty.FCBDevicePropertyMasterSlave, setMode, 0)

    @property
    def LinkState(self):
        return broadway_api.get_device_property_uint(self.__device, broadway_api.FCBDeviceProperty.FCBDevicePropertyPhyLinkState) == 1

    @LinkState.setter
    def LinkState(self, value):
        if value:
            setLink = 1
        else:
            setLink = 0   
        broadway_api.set_device_property_uint(self.__device, broadway_api.FCBDeviceProperty.FCBDevicePropertyPhyLinkState, setLink, 0)


    def get_phy_type(self):
        return self.PhyType


    def twinkle_start(self, twTime):
        self.set_led_red(0, 250, 250)

    def twinkle_stop(self):
        self.release_led_red()

    def reset_hw(self):
        broadway_api.set_device_property_uint(self.__device, broadway_api.FCBDeviceProperty.FCBDevicePropertyReset, 1, 0)

    def reset_hw_by_pin(self):
        """ reserved for test mode only"""
        broadway_api.set_pin(self.__device, broadway_api.FCBPinIds.PhyReset, 1)
        broadway_api.set_pin(self.__device, broadway_api.FCBPinIds.PhyReset, 0)

    def set_led_red(self, state, onTime, offTime):
        broadway_api.set_led_mode(self.__device, broadway_api.FCBLedId.Red, broadway_api.FCBLedControlMode.FCBLedControlMode_Host, state, onTime, offTime)

    def release_led_red(self):
        broadway_api.set_led_mode(self.__device, broadway_api.FCBLedId.Red, broadway_api.FCBLedControlMode.FCBLedControlMode_Device, 0, 0, 0)

    def query_led_red(self):
        res = broadway_api.get_led_mode(self.__device, broadway_api.FCBLedId.Red)
        return res

    def set_led_green(self, state, onTime, offTime):
        broadway_api.set_led_mode(self.__device, broadway_api.FCBLedId.Green, broadway_api.FCBLedControlMode.FCBLedControlMode_Host, state, onTime, offTime)

    def release_led_green(self):
        broadway_api.set_led_mode(self.__device, broadway_api.FCBLedId.Green, broadway_api.FCBLedControlMode.FCBLedControlMode_Device, 0, 0, 0)

    def query_led_green(self):
        res = broadway_api.get_led_mode(self.__device, broadway_api.FCBLedId.Green)
        return res

    def set_led_blue(self, state, onTime, offTime):
        broadway_api.set_led_mode(self.__device, broadway_api.FCBLedId.Blue, broadway_api.FCBLedControlMode.FCBLedControlMode_Host, state, onTime, offTime)

    def release_led_blue(self):
        broadway_api.set_led_mode(self.__device, broadway_api.FCBLedId.Blue, broadway_api.FCBLedControlMode.FCBLedControlMode_Device, 0, 0, 0)

    def query_led_blue(self):
        res = broadway_api.get_led_mode(self.__device, broadway_api.FCBLedId.Blue)
        return res

    def get_timestamp(self):
        return broadway_api.get_timestamp(self.__device)

    def init_timestamp(self, sec, nanosec):
        broadway_api.init_timestamp(self.__device, sec, nanosec)

    def update_timestamp(self, sec, nanosec):
        broadway_api.update_timestamp(self.__device, sec, nanosec)

    def adjust_ptp_clock(self, factor):
        broadway_api.adjust_ptp_clock(self.__device, factor)

    def set_friendly_name(self, friendlyName):
        """change friendly name used to easy identify multiple instances"""
        broadway_api.set_device_property_string(self.__device, broadway_api.FCBDeviceProperty.FCBDevicePropertyFriendlyName, friendlyName, 1)
        # as stick property has changed, we need to update local fcb properties

    def get_mac_address_override(self):
        """return override MAC-Address if used"""
        res = broadway_api.get_device_property_binary(self.__device, broadway_api.FCBDeviceProperty.FCBDevicePropertyMACAddressOverride)
        resString = ''
        for m in res:
             resString += '{0:02X}'.format(m)

        return resString

    def set_mac_address_override(self, macAddressStr):
        macAddressBin = bytearray(binascii.a2b_hex(macAddressStr))
        broadway_api.set_device_property_binary(self.__device, broadway_api.FCBDeviceProperty.FCBDevicePropertyMACAddressOverride, macAddressBin, 1)

    def use_mac_address_override(self, useOverride=False):
        value = 0
        if useOverride:
            value = 1
        broadway_api.set_device_property_uint(self.__device, broadway_api.FCBDeviceProperty.FCBDevicePropertyUseMACAddressOverride, value, 1)

    def trigger_wakeup(self):
        """trigger wakeup-pin on TJA110x"""
        res = broadway_api.set_device_property_uint(self.__device, broadway_api.FCBDeviceProperty.FCBDevicePropertyPhyWake, 1, 0)

        return res

    def query_enable(self):
        """query state of EN-pin connected to TJA110x"""
        res = broadway_api.query_pin(self.__device, broadway_api.FCBPinIds.PhyEnable)

        return res

    def query_inh(self):
        """query state of INH-pin connected to TJA110x"""
        res = broadway_api.query_pin(self.__device, broadway_api.FCBPinIds.PhyInh)

        return res

    def config_ext_trigger_out1(self, mode, etherTypeList, pulseWidth):
        """ configure trigger out 1: Trigger output on UART_TX, event = Rx-Frame
        mode = 0 disabled, 1=falling-edge, 2=rising-edge
        """
        output = 0  # UART_TX
        event = 0   # RX-Frame

        res = broadway_api.config_ext_trigger_out(self.__device, event, output, mode, etherTypeList, pulseWidth)
        
        return res

    def get_ext_trigger_out_status(self):
        """get current trigger_ext_out configuration"""
        return broadway_api.get_ext_trigger_out_status(self.__device)

    def force_trigger_out1(self):
        """manual trigger-out UART-TX for test/cable-check"""
        output = 0  # UART TX pin

        return broadway_api.force_trigger_out(self.__device, output)

    def set_min_tx_frame_interval(self, minInterval):
        """adjust max. rate of raw-send to limit sender"""
        return broadway_api.set_tx_frame_min_interval(self.__device, minInterval)

    def get_min_tx_frame_interval(self):
        """return current value of tx rate limiter"""
        return broadway_api.get_tx_frame_min_interval(self.__device)

    def get_frame_stats(self):
        return broadway_api.get_frame_stats(self.__device)

    def get_frame_stats_reset(self):
        return broadway_api.get_frame_stats_reset(self.__device)



